﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using CryptoPhoto;
using System.Web.UI.HtmlControls;

namespace CryptoPhoto.Test
{
    public partial class Default : System.Web.UI.Page
    {
        public static string PrivateKey = "00000000000000000000000000000000"; // Get these from https://cryptophoto.com/admin/account
        public static string PublicKey = "00000000000000000000000000000000"; // Get these from https://cryptophoto.com/admin/account
        public static int UserID = 7;
        string errorMessage;
        CryptoPhotoUtils cpUtils;
        //string UserIPAddress = "64.69.46.217";

        protected void Page_Load(object sender, EventArgs e)
        {
            // catch mobile challenge solving postback
            ProcessMobileChallengeSolve(sender);
        }

        private void ProcessMobileChallengeSolve(object sender)
        {
            if (Page.IsPostBack)
            {
                // don't execute if the user has pressed "Continue" to prevent double CryptoPhoto verification call.
                if (Request.Form["btnAuthContinue"] != null) return;

                if (ViewState["CryptoPhotoState"] != null && ViewState["CryptoPhotoState"].ToString() == "Authentication")
                {
                    // postback is a result of challenge solving via mobile app, so check authentication
                    btnAuthContinue_Click(this, EventArgs.Empty);
                }
            }
        }

        protected void btnLogin_Click(object sender, EventArgs e)
        {
            // if username and password is correct, start CryptoPhoto session
            bool isLoginValid = (txtUsername.Text.Equals("CryptoPhoto", StringComparison.OrdinalIgnoreCase)) && (txtPassword.Text.Equals("test123", StringComparison.OrdinalIgnoreCase));

            if (!isLoginValid)
            {
                errorMessage = "Incorrect username/password";
                litErrorMessage.Text = errorMessage;
                divLoginError.Visible = true;
                return;
            }

            // begin crypto photo verification procedure
            cpUtils = new CryptoPhotoUtils(PrivateKey, PublicKey);

            // get session
            List<string> sessionResponse = cpUtils.StartSession(UserID, Request.UserHostAddress, true);
            // Check if session request was successful
            bool isSessionValid = !string.IsNullOrEmpty(cpUtils.SessionID);

            if (!isSessionValid)
            {
                errorMessage = string.Format("Error: {0}", sessionResponse[1]);
                litErrorMessage.Text = errorMessage;
                divLoginError.Visible = true;
                return;
            }

            // check if user has a token generated and downloaded
            bool hasToken = bool.Parse(sessionResponse[2]);
            // display token generation widget if he hasn't
            if (!hasToken)
            {
                string genWidget = cpUtils.GetGenWidget();
                litGenWidget.Text = genWidget;
                divGen.Visible = true;
                divAuth.Visible = false;
            }
            // display auth widget if he has
            else
            {
                string authWidget = cpUtils.GetAuthWidget();
                litAuthWidget.Text = authWidget;
                divAuth.Visible = true;
                divGen.Visible = false;
                // set CryptoPhoto login state, used for the auto postback that happens when the user solves the challenge via mobile app
                ViewState["CryptoPhotoState"] = "Authentication";
            }
            divLogin.Visible = false;
            divCryptoPhoto.Visible = true;

            // save the cpUtils object inside viewstate for later use
            Session["CpUtilsObject"] = cpUtils;

            //Session["Authenticated"] = true;
            //Response.Redirect("~/protectedarea.aspx");
        }

        protected void btnGenDone_Click(object sender, EventArgs e)
        {
            Response.Redirect("~/default.aspx");
        }

        protected void btnAuthContinue_Click(object sender, EventArgs e)
        {
            cpUtils = (CryptoPhotoUtils)Session["CpUtilsObject"];

            // extract challenge fields
            string responseRow = Request.Form["token_response_field_row"];
            string responseCol = Request.Form["token_response_field_col"];
            string selector = Request.Form["token_selector"];
            string cph = Request.Form["cp_phc"];
            string btnAuth = Request.Form["btnAuthContinue"];
            // check if the user has successfully authenticated via CryptoPhoto
            List<string> verificationResponse = cpUtils.VerifyResponse(selector, responseRow, responseCol, cph, UserID, Request.UserHostAddress);

            if (verificationResponse[0] == "success")
            {
                // user has successfully authenticated
                Session["Authenticated"] = true;
                Response.Redirect("~/protectedarea.aspx");
            }
        }
    } 
}
